#include <iostream>
#include <vector>
#include <stack>
#include <algorithm>

using namespace std;

vector<int> euler(int start, vector< vector<int> > graf) {
	int n = graf.size();

	vector< vector<int> > edge(n, vector<int>(n));
	for(int u = 0; u < n; u++)
		for(int v : graf[u])
			edge[u][v]++;

	vector<int> tour;
	stack<int> s;
	s.push(start);

	while(!s.empty()) {
		int v = s.top();

		while(!graf[v].empty() && edge[v][graf[v].back()] == 0)
			graf[v].pop_back();
		
		if(graf[v].empty()) {
			tour.push_back(v);
			s.pop();
			continue;
		}

		edge[v][graf[v].back()]--;
		edge[graf[v].back()][v]--;
		s.push(graf[v].back());
		graf[v].pop_back();
	}

	return tour;
}

vector<int> eulerian(vector< vector<int> > graf, int m) {
	vector<int> nepar;
	for(int i = 0; i < graf.size(); i++)
		if(graf[i].size() % 2 == 1)
			nepar.push_back(i);

	if(nepar.size() == 0)
		return euler(0, graf);

	if(nepar.size() != 2)
		return {};

	graf[nepar[0]].push_back(nepar[1]);
	graf[nepar[1]].push_back(nepar[0]);

	vector<int> ciklus = euler(nepar[0], graf);

	if(ciklus.back() != nepar[1])
		for(int i = 0; i < m; i++)
			if(ciklus[i] == nepar[0] && ciklus[i + 1] == nepar[1]) {
				rotate(ciklus.begin(), ciklus.begin() + i + 1, ciklus.end());
				break;
			}

	return ciklus;
}

int main() {
	int n, m;
	cin >> n >> m;

	vector< vector<int> > graf(n);
	for(int i = 0; i < m; i++) {
		int u, v;
		cin >> u >> v;

		graf[u].push_back(v);
		graf[v].push_back(u);
	}

	vector<int> tour = eulerian(graf, m);
	
	if(tour.empty()) {
		cout << "Ne moze.";
		return 0;
	}
	
	for(int i = 0; i < m; i++)
		cout << '[' <<  tour[i] << ' ' << tour[i + 1] << ']';
	cout << '\n';

	return 0;
}
